"""Small example datasets.

.. autosummary::
   :toctree: .

   file_fcs
   file_jpg_paradisi05
   dir_scrnaseq_cellranger
   anndata_mouse_sc_lymph_node
   anndata_human_immune_cells
"""

from ._core import (
    anndata_human_immune_cells,
    anndata_mouse_sc_lymph_node,
    dir_scrnaseq_cellranger,
    file_fcs,
    file_jpg_paradisi05,
)
