"""Manage Jupyter notebooks.

.. autosummary::
   :toctree: .

   header
   publish

For more background, see `nbproject <https://lamin.ai/docs/nbproject>`__.
"""
from ._nb import header, publish  # noqa
